using System;
using System.Configuration;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Obrancovia
{
    /// <summary>
    /// Farba modelu hernej postavy.
    /// </summary>
    public enum FarbaPostavy
    {
        Modra,
        Zelena,
        Fialova,
        Cervena,
        Seda,
        Zlta,
        Min = Modra,
        Max = Zlta,
    }

    /// <summary>
    /// Smer otoenia hernej postavy.
    /// </summary>
    public enum SmerPostavy
    {
        Dolu,
        Hore,
        Dolava,
        Doprava
    }

    /// <summary>
    /// Trieda hernej postavy.
    /// </summary>
    public class Player : IDisposable
    {
        #region Kontanty
        // Konstanty pre kreslenie
        private const float KonstantaZvacsenia = 0.1f;
        private static readonly Matrix[] MaticaOtocenia = new Matrix[] {
            Matrix.RotationY((float)Math.PI * 3.0f / 2.0f),
            Matrix.RotationY((float)Math.PI / 2.0f),
            Matrix.RotationY((float)Math.PI * 2),
            Matrix.RotationY((float)Math.PI) };

        private static readonly Matrix MaticaZvacsenia = Matrix.Scaling(
            KonstantaZvacsenia, KonstantaZvacsenia, KonstantaZvacsenia);

        // Maximlna rchlos pohybu hernej postavy.
        private const float MaximalnaRychlost = 30.0f;
        // Maximlna rchlos skoku.
        private const float RychlostSkoku = 10.0f;
        private const float MaximalnaVyskaSkoku = 0.4f;

        #endregion

        #region Premenne triedy
        private Mesh meshPostavy = null; // Mesh pre hern postavu
        private Material materialPostavy; // Materil
        private Texture texturaPostavy = null; // Textra
        private Matrix otocenia = MaticaOtocenia[0];

        private Vector3 umiestnenie; // Umiestnenie hernej postavy.
        private Vector3 posunNaPoziciu; // Kam presvam hern postavu.
        private bool jeVPohybe = false;
        private float vyskaPostavy = 0.0f;
        #endregion 

        #region Kontruktor
        /// <summary>
        /// Vytvor nov instanciu hernej postavy.
        /// </summary>
        public Player(Mesh meshLoopyho, Texture texturaLoopyho,
          Material material)
        {
            // Ulom mesh
            meshPostavy = meshLoopyho;
            
            // Ulom textru
            texturaPostavy = texturaLoopyho;

            // Ulom materil
            materialPostavy = material;
        }
        #endregion

        #region Vlastnosti triedy
        /// <summary>
        /// Pozcia hernej postavy.
        /// </summary>
        public Vector3 Umiestnenie
        {
            get { return umiestnenie; }
            set { umiestnenie = posunNaPoziciu = value; }
        }
        #endregion

        #region Metody triedy
        public void Draw(Device zariadenie, float appCas)
        {
            // Nastavm transformciu sveta.
            zarizeni.Transform.World = otocenie * MaticaZvacsenia *
              Matrix.Translation(umisteni.X, vyskaPostavy, umiestnenie.Z);

            // Nastavm texturu modelu.
            zariadenie.SetTexture(0, texturaPostavy);
            // Nastavm materil modelu
            zariadenie.Material = materialPostavy;

            // Zobrazm model.
            meshPostavy.DrawSubset(0);
        }

        /// <summary>
        /// Nastav smer otoenia hernej postavy.
        /// </summary>
        public void NastavSmer(SmerPostavy smer)
        {
            otocenie = MaticaOtocenia[(byte)smer];
        }
        /// <summary>
        /// Postupne presunie hern postavu na nov pozciu.
        /// </summary>
        /// <param name="newPosition">Nov pozcia hernej postavy</param>
        public bool PosunNa(Vector3 noveUmiestnenie)
        {
            if (!jeVPohybe)
            {
                jeVPohybe = true;
                // Ulom nov poziciu, ktor bude pouit behom presunu.
                posunNaPoziciu = noveUmiestnenie;
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <param name="uplynuly">as, ktor uplynul od poslednho snmku</param>
        /// <param name="celkovy">celkov as</param>
        public void Aktualizuj(float uplynuly, float celkovy)
        {
            // Vypotme nov pozciu hernej postavy.
            vyskaPostavy = (float)(Math.Abs(Math.Sin(celkovy * RychlostSkoku))) *
              MaximalnaVyskaSkoku;

            if (umiestnenie != posunNaPoziciu)
            {
                Vector3 diff = posunNaPoziciu - umiestnenie;
                // S tieto pozce tak blzko, e meme rovno prejs z jednej na
                // druh?
                if (diff.LengthSq() > (MaximalnaRychlost * uplynuly))
                {
                    // Niesom dostatene blzko, musm prejs postupne.
                    diff.Normalize();
                    diff.Scale(MaximalnaRychlost * uplynuly);
                    umiestnenie += diff;
                }
                else
                {
                    jeVPohybe = false;
                    // Naozaj som blzko, prejdem hne.
                    umiestnenie = posunNaPoziciu;
                }
            }
            else
            {
                jeVPohybe = false;
            }
        }

        #endregion

        #region Rozhranie IDisposable 

        /// <summary>
        /// Upratovanie v prpade, e Dispose nebolo zavolan.
        /// </summary>
        ~Player()
        {
            Dispose();
        }
        public void Dispose()
        {
            GC.SuppressFinalize(this);

            // Uvonme textru.
            if (texturaPostavy != null)
            {
                texturaPostavy.Dispose();
            }

            if (meshPostavy != null)
            {
                meshPostavy.Dispose();
            }

            texturaPostavy = null;
            meshPostavy = null;
        }

        #endregion
    }
}